/*
    Copyright 2009 Graham King <graham@gkgk.org>
    
    This file is part of FireBucket.

    FireBucket is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FireBucket is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FireBucket.  If not, see <http://www.gnu.org/licenses/>.
*/

if (typeof FB == "undefined" || !FB) {
	var FB = {};
}

FB.OBJ = {
			
	EXTENSION_ID: "firebucket@darkcoding.net",
	
	PATH_SEP: null,
	
	// Headers S3 won't allow you to change
	RESTRICTED_HEADERS: ['x-amz-request-id', 'x-amz-id-2', 'Etag', 
						'Content-Length', 'Server', 'Date', 
						'Last-Modified'],
	
	setup:
		function() {
	
			var path = Components.classes['@mozilla.org/extensions/manager;1'].
								getService(Components.interfaces.nsIExtensionManager).
								getInstallLocation(FB.OBJ.EXTENSION_ID).
								getItemLocation(FB.OBJ.EXTENSION_ID).path;
	
			if (path.search(/\\/) != -1) {
				FB.OBJ.PATH_SEP = "\\";
			} 
			else {
				FB.OBJ.PATH_SEP = "/";
			}
			
		},
	
	log: 
		function(msg) {
    		dump(msg +'\n');   
		},
		
	status:
		function(msg) {
			var statusBox = document.getElementById('status');
			if ( msg == '' ) {
				statusBox.className = 'hidden';
			}
			else {
				statusBox.className = '';
				statusBox.innerHTML = msg;
			}
		},
	
	properties:
		function(obj) {
			var props = '';
			for (var p in obj) { props += p +', ' };
			props = props.slice(0, props.length-2);
			FB.OBJ.log(props);
		},
		
	dumpObj:
		function(obj) {
			for (var p in obj) {
				FB.OBJ.log(p +': '+ obj[p]);
			}
		},
		
	escapeXML:
		function(str) {
			return str.replace(/&/g, '&amp;').replace(/\'/g, '&apos;').replace(/\"/g, '&quot;');
		},
		
	escapeURL:
		function(url) {
			return escape(url).replace(/%3A/, ':');
		},
		
	directory:
		function(name) {
			var extension = Components.classes['@mozilla.org/extensions/manager;1'].
								getService(Components.interfaces.nsIExtensionManager).
								getInstallLocation(FB.OBJ.EXTENSION_ID).
								getItemLocation(FB.OBJ.EXTENSION_ID);
			var dirname = extension.path + FB.OBJ.PATH_SEP + 
							'chrome' + FB.OBJ.PATH_SEP + 
							'content' + FB.OBJ.PATH_SEP + 
							name;
			var dir = DirIO.open(dirname);
			if ( ! dir.exists() ) {
				dir.create(0x01, 0774);
			}
			return dir;
		},
		
	storeDir:
		function() {
			return this.directory('store');
		},
		
	cacheDir:
		function() {
			return this.directory('cache');
		},
		
	getCharPref:
		function(prefName) {
			var prefManager = Components.classes["@mozilla.org/preferences-service;1"].
								getService(Components.interfaces.nsIPrefBranch);
			try {
				return prefManager.getCharPref('extensions.firebucket.'+ prefName);
			}
			catch (e) {
				return null;
			}
		},
		
	setCharPref:
		function(prefName, value) {
			var trimmedValue = FB.OBJ.trim(value);
			var prefManager = Components.classes["@mozilla.org/preferences-service;1"].
								getService(Components.interfaces.nsIPrefBranch);
			return prefManager.setCharPref('extensions.firebucket.'+ prefName, trimmedValue);
		},
		
	mimeType:
		function(filename) {

			var ext = filename.slice( filename.length-3, filename.length ); 

			var mimeService = Components.classes["@mozilla.org/mime;1"].
								getService(Components.interfaces.nsIMIMEService);
			try {
				var mimeInfo = mimeService.getTypeFromExtension(ext);
				return mimeInfo;
			}
			catch (e) {
				return "binary/octet-stream";
			}
		},
		
	extractKey:
		function(url) {
			var no_scheme_url = url.slice( 'http://'.length, url.length );
			var key = no_scheme_url.slice( no_scheme_url.indexOf('/')+1, no_scheme_url.length );
			return key;
		},
		
	trim:
		function(value) {
			return value.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
		},
		
	contains:
		function contains(arr, obj) {
  			var i = arr.length;
  			while (i--) {
    			if (arr[i] === obj) {
      				return true;
    			}
  			}
  			return false;
		},
		
	open:
		function(url) {
			FB.OBJ._openAndReuseOneTabPerAttribute('firbucket', url);
		},
		
	_openAndReuseOneTabPerAttribute:	// https://developer.mozilla.org/en/Code_snippets/Tabbed_browser
		function(attrName, url) {
		  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
		           .getService(Components.interfaces.nsIWindowMediator);
		  for (var found = false, index = 0, tabbrowser = wm.getEnumerator('navigator:browser').getNext().getBrowser();
		       index < tabbrowser.mTabs.length && !found;
		       index++) {
		
		    // Get the next tab
		    var currentTab = tabbrowser.mTabs[index];
		  
		    // Does this tab contain our custom attribute?
		    if (currentTab.hasAttribute(attrName)) {
		
		      // Yes--select and focus it.
		      tabbrowser.selectedTab = currentTab;
		
		      // Focus *this* browser in case another one is currently focused
		      tabbrowser.loadURI(url, tabbrowser.currentURI, "UTF-8");
		      tabbrowser.focus();

		      found = true;
		    }
		  }
		
		  if (!found) {
		    // Our tab isn't open. Open it now.
		    var browserEnumerator = wm.getEnumerator("navigator:browser");
		    var tabbrowser = browserEnumerator.getNext().getBrowser(); 
		  
		    // Create tab
		    var newTab = tabbrowser.addTab(url);
		    newTab.setAttribute(attrName, "xyz");
		  
		    // Focus tab
		    tabbrowser.selectedTab = newTab;
		    
		    // Focus *this* browser in case another one is currently focused
		    tabbrowser.focus();
		  }
		},
		
	isAllowedHeader:
		function(candidate) {
			return ! FB.OBJ.contains(FB.OBJ.RESTRICTED_HEADERS, candidate);
		},

	isoDateTime:
		function(d) {

			function pad(c) {
				var c = new String(c);
				if (c.length < 2) {
					c = '0'+ c;
				}
				return c;
			}

			if (!d) {
				d = new Date();
			}

			var iso = d.getUTCFullYear() +'-'+ pad(d.getUTCMonth()) +'-'+ pad(d.getUTCDay()) +'T'+ 
					pad(d.getUTCHours()) +':'+ pad(d.getUTCMinutes()) +':'+ pad(d.getUTCSeconds()) +'.000Z';
					
			return iso;
		}
	
};


FB.OBJ.Account = function (keyId, secretId) {

	this.buckets = [];

	S3Ajax.KEY_ID = keyId;
    S3Ajax.SECRET_KEY = secretId;	
}

FB.OBJ.Account.prototype = {

	loadBuckets:
		function(topCallback, forceReload) {

			function addBucket(bucketName) {
				var bucketObj = new FB.OBJ.Bucket(bucketName);
				_this.buckets.push(bucketObj);
			}
		
			function successCallback(req, obj) {

				var bucketListStr = "";
		        var buckets = obj.ListAllMyBucketsResult.Buckets.Bucket;
		        for (var i=0, bucket; bucket=buckets[i]; i++) {
		        	addBucket(bucket.Name);
		        	bucketListStr += bucket.Name +',';
		        }

				bucketListStr = bucketListStr.slice(0, bucketListStr.length-1);
				FB.OBJ.setCharPref('bucketList', bucketListStr);
				
				FB.OBJ.status('');
				topCallback();
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.status('Error '+ req.status +' loading buckets. '+ req.statusText);
		    }
		    
		    var _this = this;
			this.buckets = [];
			
		    var bucketListStr = FB.OBJ.getCharPref('bucketList');
		    if ( ! forceReload && bucketListStr != null && bucketListStr.length > 0 ) {
	
		    	var bucketList = bucketListStr.split(',');
		    	for (var i=0; i < bucketList.length; i++) {
		    		addBucket(bucketList[i]);
		    	}
		    	topCallback();
		    }
		    else {
			    FB.OBJ.status('Loading buckets...');
		    	S3Ajax.listBuckets(successCallback, errorCallback);
		    }
		},
			
	isBucketNameAvailable:
		function(name, callback) {
		
			function successCallback(req, obj) {
				callback(false);		// Bucket exists, we own it
		    }
		    
		    function errorCallback(req) {
		    	if ( req.status == 403 ) {
		    		callback(false);	// Bucket exists, someone else owns it
		    	}
		    	else if ( req.status == 404 ) {
		    		callback(true);		// Name is available
		    	}
		    	else {
		    		// A real error
			        FB.OBJ.status('Error '+ req.status +' loading buckets. '+ req.statusText);
		    	}
		    }
		
			S3Ajax.checkBucket(name, successCallback, errorCallback);
		},
		
		createBucket:
			function(name, location, acl, callback) {
			
				function successCallback(req, obj) {
					callback();
			    }
			    
			    function errorCallback(req) {
			        FB.OBJ.status('Error '+ req.status +' loading buckets. '+ req.statusText);
			        //callback();
			    }

				S3Ajax.createBucket(name, location, acl, successCallback, errorCallback);
			}
		
};

FB.OBJ.Bucket = function(bucketName) {
	this.name = bucketName;
	this.location = null;
	
	var locPref = this.name +'.location';
	this.location = FB.OBJ.getCharPref(locPref);
	if (this.location == null) {
	
		function setLocation(loc) {
			_this.location=loc; 
			FB.OBJ.setCharPref(locPref, _this.location);
		}
		var _this = this;
		this._loadLocation(setLocation);
	}
}

FB.OBJ.Bucket.prototype = {

	listItems:
		function(topCallback, forceReload) {

			var _this = this;
	
			var file = FileIO.open( this.storeURL() );
			if ( file.exists() && ! forceReload ) {
				topCallback();
			}
			else {
				FB.OBJ.status('Listing bucket items...');
				
				var downloadManager = new FB.OBJ.DownloadManager(this, topCallback);
				downloadManager.start();
		    }   
		},

	_loadLocation:
		function(callback) {
			function successCallback(req, obj) {
				var loc = obj.LocationConstraint;
				if ( loc == '' ) {
					loc = 'US';
				}
				callback(loc);
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.log('Error '+ req.status +' getting ACL. '+ req.statusText);
		    }
		    
			S3Ajax.getBucketLocation(this.name, successCallback, errorCallback);
		},

	getACL:
		function(callback) {
			function successCallback(req, obj) {
				callback(obj, req.responseText);
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.log('Error '+ req.status +' getting ACL. '+ req.statusText);
		        if (_this.location == 'EU') {
		        	callback(null, 
		        			"Fetching this buckets ACL failed. As this is an EU bucket, this could "+
		        			"be because it hasn't propagated yet. Please try again in 5 minutes.");
		        }
		    }

			var _this = this;
			S3Ajax.getBucketACL(this.name, successCallback, errorCallback);
		},

	setACL:
		function(xmlACL) {
			function successCallback(req, obj) {
				FB.OBJ.status('A.C.L. updated');
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.log('Error '+ req.status +' setting acl. '+ req.statusText);
		    }

			S3Ajax.setBucketACL(this.name, xmlACL, successCallback, errorCallback);
		},

	getLogging:
		function(callback) {
	
			function successCallback(req, obj) {
				callback(obj);
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.log('Error '+ req.status +' getting logging. '+ req.statusText);
		    }
	
			S3Ajax.getBucketLogging(this.name, successCallback, errorCallback);
		},

	setLoggingOn:
		function(targetBucket, targetPrefix) {

			function successCallback(req, obj) {
				FB.OBJ.status('Bucket logging updated');
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.status('Error '+ req.status +' setting logging on. '+ req.statusText);
		    }

			var xml = FB.OBJ.XML.LOGGING_ON.replace(/##targetBucket##/, targetBucket);
			xml = xml.replace(/##targetPrefix##/, targetPrefix);
			
			S3Ajax.setBucketLogging(this.name, xml, successCallback, errorCallback);
		},

	setLoggingOff:
		function() {

			function successCallback(req, obj) {
				FB.OBJ.status('Bucket logging updated');
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.status('Error '+ req.status +' setting logging off. '+ req.statusText);
		    }

			S3Ajax.setBucketLogging(this.name, FB.OBJ.XML.LOGGING_OFF, successCallback, errorCallback);
		},

	upload:
		function(key, mimeType, headers, file, callback) {
		
			function successCallback(req, obj) {
				FB.OBJ.status('Upload succeeded');
				
				var item = new FB.OBJ.Item(_this, 'http://'+ _this.name +'.s3.amazonaws.com/'+ key);
				FB.OBJ.RDF.addNode(item);
				
				callback();
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.status('Error '+ req.status +' uploading. '+ req.statusText);
		    }

			var _this = this;

			var meta = {};
			var extraHeaders = {};
			for (var prop in headers) {
				if (prop.indexOf('x-amz') == 0) {
					meta[prop] = headers[prop];
				}
				else {
					extraHeaders[prop] = headers[prop];
				}
			}

			var fileData = FileIO.readBinary(file);
			var params = {};
			params.content_type = mimeType;
			params.meta = meta;
			params.extraHeaders = extraHeaders;
			
			S3Ajax.put(
						this.name, 
						key, 
						fileData, 
						params,
						successCallback, 
						errorCallback);
		},

	delete:
		function(callback) {

			function successCallback(req, obj) {
				FB.OBJ.status('Bucket deleted');
				callback();
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.status('Error '+ req.status +' deleting bucket. '+ req.statusText);
		    }

			S3Ajax.deleteBucket(this.name, successCallback, errorCallback);
		},

	storeURL:
		function() {
			return FB.OBJ.storeDir().path +FB.OBJ.PATH_SEP+ this.name +'.rdf'
		},
		
	loadItem: 
		function(url, topCallback) {
			var item = new FB.OBJ.Item(this, url);
			if ( item.isLoaded ) {
				topCallback(item);
			}
			else {
				FB.OBJ.status('Fetching headers...');
				item.fetchAndCacheHeaders(
					function() {
						FB.OBJ.status('Downloading...');
						item.fetchAndCacheFile(topCallback);
					}
				);				
			}
		},
		
	copy:
		function(item, destKey, headers, topCallback) {
			
			function successCallback(req, obj) {
				FB.OBJ.status('Copy succeeded');
				topCallback();
		    }
			
		    function errorCallback(req) {
			    var err = 'Error '+ req.status +' copying. '+ req.statusText;
		        FB.OBJ.status(err);
		        FB.OBJ.log(err);
		        topCallback();
		    }

			headers['x-amz-copy-source'] = '/'+ this.name +'/'+ item.key
	   	    if (item.key == destKey) {
	       	    headers['x-amz-metadata-directive'] = 'REPLACE'
			}

			var params = {};
			params.content_type = headers['Content-Type'];
			delete headers['Content-Type'];
			
			var meta = {};
			var extraHeaders = {};
			for (var prop in headers) {
				if (prop.indexOf('x-amz') == 0) {
					meta[prop] = headers[prop];
				}
				else {
					extraHeaders[prop] = headers[prop];
				}
			}

			params.meta = meta;
			params.extraHeaders = extraHeaders;
			
			S3Ajax.put(
						this.name, 
						destKey, 
						null, 
						params,
						successCallback, 
						errorCallback);
		},
		
	/* Iterate through every element in this bucket and set it's Content-Type type
	 * to what Firefox thinks it should be.
	 */
	fixAllMimeTypes:
		function() {
		
			function onComplete() {
				FB.OBJ.status('Operation completed');
			}
		
			var _this = this;
			
			var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"]
									.getService(Components.interfaces.nsIRDFService);
			
			var containerUtils = Components.classes["@mozilla.org/rdf/container-utils;1"]
									.getService(Components.interfaces.nsIRDFContainerUtils);
			
			var ds = RDF.GetDataSource("file://"+ this.storeURL());
			var allResources = ds.GetAllResources();
			
			var urlList = [];
			
			while (allResources.hasMoreElements()) {
				var next = allResources.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
				
				if ( containerUtils.IsContainer(ds, next) ) {
					continue;
				}
				
				urlList.push(next.Value);
			}
			
			var proc = new FB.OBJ.SequentialProcessor(
										urlList, 
										function(item, callback) {_this.fixIndividualMimeType(item, callback)}, 
										onComplete);
			proc.start();
		},
		
	fixIndividualMimeType:
		function(itemURL, callback) {
		
			function nextStep() {
				if (item.headers) {
					item.fixMimeType( callback );
				}
				else {
					callback();
				}
			}
						
			var item = new FB.OBJ.Item(this, itemURL);
			if (item.isLoaded) {
				nextStep();
			}
			else {
				item.fetchAndCacheHeaders(nextStep);
			}
		}
		
};

FB.OBJ.Item = function(bucket, url) {

	var _this = this;
		
	this.bucket = bucket;
	this.url = url;
	this.headers = null;
	this.key = FB.OBJ.extractKey(url);
	this.name = (function() { var spl = _this.key.split('/'); return spl[spl.length - 1] }());
	this.itemCacheFile = null;
	this.headersCacheFile = null;
	this.isLoaded = false;
	
	this.etag = "TODO";	// This is one of the headers, or we get it as a response to an upload
	
	if ( this.key.indexOf('/') != -1 ) {
		var dirPart = this.key.slice(0, this.key.lastIndexOf('/'));
		dirPart = dirPart.replace(/\//g, FB.OBJ.PATH_SEP);		// Just in case Windows
		FB.OBJ.directory('cache' + FB.OBJ.PATH_SEP + dirPart);	// Creates the necessary subdirs of cache
	}

	var keyAsPath = this.key.replace(/\//g, FB.OBJ.PATH_SEP);
	
	this.headersCacheFile = FileIO.open(FB.OBJ.cacheDir().path +FB.OBJ.PATH_SEP+ keyAsPath +'.headers');
	this.itemCacheFile = FileIO.open(FB.OBJ.cacheDir().path +FB.OBJ.PATH_SEP+ keyAsPath);
	
	if (this.itemCacheFile.exists() && this.headersCacheFile.exists()) {
		this.isLoaded = true;
		this.headers = FileIO.read(this.headersCacheFile);
	}
	
};

FB.OBJ.Item.prototype = {

	fetchAndCacheHeaders:
		function(callback) {
			var _this = this;
			var req = new XMLHttpRequest();
			req.open('HEAD', FB.OBJ.escapeURL(this.url), true);
			
			req.onreadystatechange = function (event) {  
				if (req.readyState == 4) {  
					if(req.status == 200)  {
						_this.headers = req.getAllResponseHeaders();
						FileIO.write(_this.headersCacheFile, _this.headers);
						FB.OBJ.status('');
						callback();
					}
					else  {
						FB.OBJ.log('Error fetching headers for: '+ _this.url);
				        FB.OBJ.status('Remote error '+ req.status +' loading item headers. '+ req.statusText);
				        callback();
					}
				}
			};
			
			req.send(null);
		},
	
	fetchAndCacheFile:
		function(callback) {
			var _this = this;
			var progressListener = {
		
	  			onProgressChange: 
	  				function(webProgress, req, curSelfProg, maxSelfProg, curTotalProg, maxTotalProg) {
	    				var pctDone = (curTotalProg / maxTotalProg) * 100;
		    			document.getElementById("fb_dl_progress").setAttribute("value", pctDone);
	    				document.getElementById("fb_dl_label").
	    							setAttribute("value", "Downloading..."+ Math.round(pctDone) +"%");
	  				},
	  				
	  			onStateChange: 
	  				function(aWebProgress, aRequest, stateFlags, status) {
	
	    				if (stateFlags & Components.interfaces.nsIWebProgressListener.STATE_STOP) {
		    				FB.OBJ.status('');
							callback(_this);
	    				}
	  				}
			}

			var persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"]
	                		.createInstance(Components.interfaces.nsIWebBrowserPersist);
			var objURI = Components.classes["@mozilla.org/network/io-service;1"]
	                		.getService(Components.interfaces.nsIIOService)
	                		.newURI(FB.OBJ.escapeURL(this.url), null, null);
			persist.progressListener = progressListener;
			persist.saveURI(objURI, null, null, null, "", this.itemCacheFile);
		},

	toString:
		function() {
			return this.key +' in bucket '+ this.bucket.name;
		},

	cacheFileURL:
		function() {
			return FileIO.path(FB.OBJ.cacheDir()) +'/'+ escape(this.key);
		},

	delete:
		function(callback) {
		
			function successCallback(req, obj) {
			
				FB.OBJ.RDF.deleteNode(_this);
				FB.OBJ.status('Deleted');
				callback();
		    }
		    
		    function errorCallback(req) {
		        FB.OBJ.status('Error '+ req.status +' during delete. '+ req.statusText);
		    }
		
			var _this = this;
		
			FB.OBJ.status('Deleting '+ this.toString() +'...');
			S3Ajax.deleteKey(this.bucket.name, this.key, successCallback, errorCallback);
		},
		
	setHeaders:
		function(headers, callback) {
			this.bucket.copy(this, this.key, headers, callback);
		},
		
	headersMap:		// item headers as a hashmap
		function() {
			var result = {};
			
			if (!this.headers) {
				return result;
			}
			
			var headerRows = this.headers.split(/\n/);
			for (var i=0; i<headerRows.length; i++) {
				if (headerRows[i] == null || headerRows[i].length == 0) {
					continue;
				}
				var keyValue = headerRows[i].split(':');
				result[ keyValue[0] ] = FB.OBJ.trim(keyValue[1]);
			}
			
			return result;
		},
		
	fixMimeType:
		function(callback) {
			
			var headers = this.headersMap();
			FB.OBJ.log(this.key +" currently: "+ headers['Content-Type']);
			var correctMimeType = FB.OBJ.mimeType(this.key);
			if (headers['Content-Type'] == correctMimeType) {
				FB.OBJ.log('Skipping '+ this.key +' beause it already has the correct type');
				callback();
				return;
			}
			
			delete headers['Content-Length'];
			headers['Content-Type'] = correctMimeType;
			this.bucket.copy(this, this.key, headers, callback);
		},
		
	fileSize:
		function() {
			return 42;
		},
		
	parentURL: 
		function() {
			return this.url.slice(0, this.url.lastIndexOf('/'));
		}
};

FB.OBJ.SequentialProcessor = function(items, action, callback) {

	this.items = items;
	this.action = action;
	this.onComplete = callback;
	
	this.position = 0;
}

FB.OBJ.SequentialProcessor.prototype = {

	start:
		function() {
			this._continue();
		},
		
	_continue:
		function() {
		
			function callback() {
				_this.position++;
				
				if (_this.position < _this.items.length) {
					_this._continue();
				}
				else {
					_this.onComplete();
				}	
			}
						
			var _this = this;
			var item = this.items[this.position];
			FB.OBJ.status('Processing item '+ this.position +'...');
			this.action(item, callback);
		}

}

FB.OBJ.DownloadManager = function(bucket, callback) {

	this.bucket = bucket;
	this.callback = callback;
	
	this.step = null;
	this.lastKey = null;
	this.totalDownloaded = 0;
	this.s3Obj = null;
	this.store = {};
};

FB.OBJ.DownloadManager.ITEM_BATCH_SIZE = 1000;	// 1000 is the max batch size S3 allows
FB.OBJ.DownloadManager.MAX_ITEMS = -1; // A positive number will limit the number of items loaded from S3
FB.OBJ.DownloadManager.DOWNLOADED = 1;
FB.OBJ.DownloadManager.STORED = 2;
FB.OBJ.DownloadManager.ALL_FETCHED = 3;
FB.OBJ.DownloadManager.RDFED = 4;

FB.OBJ.DownloadManager.prototype = {

	start:
		function() {
			this._continue();
		},

	_continue: 
		function() {

			var _this = this;
			var constant = FB.OBJ.DownloadManager;
			
			var isMaxItemsReached = (constant.MAX_ITEMS > 0 && this.totalDownloaded >= constant.MAX_ITEMS);
			if (this.step == constant.STORED && isMaxItemsReached) {
				this.step = constant.ALL_FETCHED;
			}
			
			if (this.step == null || this.step == constant.STORED) {
			
				var batchEnd = this.totalDownloaded + FB.OBJ.DownloadManager.ITEM_BATCH_SIZE;
				FB.OBJ.status('Downloading items '+ this.totalDownloaded +' to '+ batchEnd +'...');
				
				setTimeout( function() {_this._download()}, 1 );
			}
			else if (this.step == constant.DOWNLOADED) {
			
				FB.OBJ.status('Converting response to object...');
				setTimeout( function() {_this._store()}, 1 );
			}
			else if (this.step == constant.ALL_FETCHED) {
			
				FB.OBJ.status('Writing RDF file...');
				setTimeout( function() {_this._writeRDF()}, 1 );
			}
			else if (this.step == constant.RDFED) {
				this.callback();
			}
		},

	_download:
		function() {
			
			function onListKeySuccess(req, obj) {
				_this.s3Obj = obj;
				_this.totalDownloaded += FB.OBJ.DownloadManager.ITEM_BATCH_SIZE;
				
				_this.step = FB.OBJ.DownloadManager.DOWNLOADED;
				_this._continue();
		    }
		    
		    function onListKeysError(req, obj) {
		        FB.OBJ.log('_download: Failed, retrying');
				FB.OBJ.status('Remote error '+ req.status +' loading items. '+ req.statusText +'. Retrying in 2 secs');
		        setTimeout(function() { _this._continue()}, 2000);
		    }
		    
			var _this = this;
			
			var s3Args = {'max-keys': FB.OBJ.DownloadManager.ITEM_BATCH_SIZE};
			if (this.lastKey != null) {
				s3Args['marker'] = this.lastKey;
			}
			
			S3Ajax.listKeys(
	    				this.bucket.name, 
	    				this.bucket.location,
	    				s3Args,
	    				onListKeySuccess, 
	    				onListKeysError
	    				);
		},

	_store:	
		function() {

			function addToStore(store, key, size, lastModified, etag, storageClass, ownerID,  ownerName) {
				var elements = key.split('/');
				
				var last = store;
				for (var i = 0; i < elements.length - 1; i++) {
					if (last[elements[i]] === undefined) {
						last[elements[i]] = {};
					}
					last = last[elements[i]];
				}
				var props = {
							'ITEM': true, 
							'size': size,
							'lastModified': lastModified,
							'etag': etag,
							'storageClass': storageClass,
							'ownerID': ownerID,
							'ownerName': ownerName
							};
				last[ elements[elements.length-1] ] = props;
			} // end addToStore

			var _this = this;

			var contents = this.s3Obj.ListBucketResult.Contents;
			if ( contents == null ) {
				// Bucket is empty
				this.step = FB.OBJ.DownloadManager.ALL_FETCHED;
				this._continue();
				return;
			}
			
			for (var i=0; i < contents.length; i++) {
				addToStore(
							this.store, 
							FB.OBJ.escapeXML(contents[i].Key), 
							contents[i].Size,
							contents[i].LastModified, 
							contents[i].ETag, 
							contents[i].StorageClass, 
							contents[i].Owner.ID, 
							contents[i].Owner.DisplayName
							);
			}

			this.lastKey = contents[contents.length-1].Key;
			
			if ( contents.length < FB.OBJ.DownloadManager.ITEM_BATCH_SIZE ) {
				this.step = FB.OBJ.DownloadManager.ALL_FETCHED;
			}
			else {
				this.step = FB.OBJ.DownloadManager.STORED;
			}
			this._continue();
		},

		_writeRDF: 
			function() {

				var rdfXML = this._toRDF(this.bucket.name, this.store);
				var file = FileIO.open( FB.OBJ.storeDir().path +FB.OBJ.PATH_SEP+ this.bucket.name +'.rdf' );
				FileIO.write(file, rdfXML);

				this.step = FB.OBJ.DownloadManager.RDFED;
				this._continue();
			},

		_itemURL:
			function(bucket, parent, prop) {
				var itemURL = FB.OBJ.RDF.S3_ITEM_URL.replace(/##bucket##/, bucket);
				itemURL = itemURL.replace(/##parent##/, parent);
				itemURL = itemURL.replace(/##key##/g, prop);
				return itemURL;
			},

		_toRDF: 
			function(bucket, store) {

				function internalToRDFItems(bucket, parent, element) {
					
					var rdf = "";
					for (var prop in element) {

						var itemURL = _this._itemURL(bucket, parent, prop);
						
						if (element[prop].ITEM === undefined) {
						
							var rdfElement = FB.OBJ.RDF.DIR.replace(/##itemURL##/, itemURL);
							rdf += rdfElement.replace(/##key##/g, prop);
							
							rdf += internalToRDFItems(bucket, parent + prop +'/', element[prop]);
						}
						else {
						
							var kSize = Math.floor(element[prop].size / 1024);
					    	if (kSize < 1) {
						 		kSize = 1;
						   	}

							var rdfItem = FB.OBJ.RDF.ITEM.replace(/##itemURL##/, itemURL);
							rdfItem = rdfItem.replace(/##key##/g, prop);
							rdfItem = rdfItem.replace(/##size##/, kSize);
							rdfItem = rdfItem.replace(/##lastModified##/, element[prop].lastModified);							
							rdfItem = rdfItem.replace(/##etag##/, element[prop].etag);
							rdfItem = rdfItem.replace(/##storageClass##/, element[prop].storageClass);
							rdfItem = rdfItem.replace(/##ownerID##/, element[prop].ownerID);
							rdfItem = rdfItem.replace(/##ownerName##/, element[prop].ownerName);
							rdf += rdfItem;
						}
					}
			
					return rdf;
				} // end internalToRDFItems
				
				function internalToRDFSequence(bucket, parent, element) {
			
					var rdf = "";
					
					for (var prop in element) {
						
						var itemURL = _this._itemURL(bucket, parent, prop);
						
						if (element[prop].ITEM == undefined) {
							rdf += "<RDF:li>\n";
							rdf += FB.OBJ.RDF.SEQ.replace(/##itemURL##/, itemURL);
							rdf += internalToRDFSequence(bucket, parent + prop +'/', element[prop]);
							rdf += FB.OBJ.RDF.SEQ_CLOSE;
							rdf += "</RDF:li>\n";
						}
						else {
							rdf += FB.OBJ.RDF.LI_RESOURCE.replace(/##itemURL##/, itemURL);
						}
					}
					
					return rdf;
				} // end internalToRDFSequence
				
				var _this = this;
				
				var result = FB.OBJ.RDF.HEAD.replace(/##bucket##/, bucket);
				
				result += internalToRDFItems(bucket, "", store);
				
				result += FB.OBJ.RDF.SEQ.replace(/##itemURL##/, _this._itemURL(bucket, "", "all"));
				result += internalToRDFSequence(bucket, "", store);
				result += FB.OBJ.RDF.SEQ_CLOSE;
				
				result += FB.OBJ.RDF.END;
				
				return result;
			}
};

/**
 * Strings used to create an RDF document as XML.
 * Singleton object literal.
 */
FB.OBJ.RDF = {};
FB.OBJ.RDF.S3_ITEM_URL = "http://##bucket##.s3.amazonaws.com/##parent####key##";
FB.OBJ.RDF.HEAD = (function() {
			var head = "<?xml version=\"1.0\"?>\n";
			head += "<RDF:RDF xmlns:RDF=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" ";
			head += "xmlns:B=\"http://##bucket##.s3.amazonaws.com/#\">\n";
			return head;
		  })();
FB.OBJ.RDF.DIR = (function() {
			var dir = "<RDF:Description RDF:about=\"##itemURL##\">\n";
			dir += "\t<B:name>##key##</B:name>\n";
			dir += "</RDF:Description>\n";
			return dir;	
		})();
FB.OBJ.RDF.ITEM = (function() {
			var item = "<RDF:Description RDF:about=\"##itemURL##\">\n";
			item += "\t<B:name>##key##</B:name>\n";
			item += "\t<B:size>##size##</B:size>\n";
			item += "\t<B:lastModified>##lastModified##</B:lastModified>\n";
			item += "\t<B:etag>##etag##</B:etag>\n";
			item += "\t<B:storageClass>##storageClass##</B:storageClass>\n";
			item += "\t<B:ownerID>##ownerID##</B:ownerID>\n";
			item += "\t<B:ownerName>##ownerName##</B:ownerName>\n";
			item += "</RDF:Description>\n";
			return item;
		})();
FB.OBJ.RDF.SEQ = "<RDF:Seq RDF:about=\"##itemURL##\">\n";
FB.OBJ.RDF.SEQ_CLOSE = "</RDF:Seq>\n";
FB.OBJ.RDF.LI_RESOURCE = "<RDF:li RDF:resource=\"##itemURL##\"/>\n";
FB.OBJ.RDF.END = "</RDF:RDF>\n";

FB.OBJ.RDF.addNode = 
	function(item) {
		var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"]
										.getService(Components.interfaces.nsIRDFService);
		var ds = RDF.GetDataSource("file://"+ item.bucket.storeURL());
		
		// Describe the new item
		
		var resource = RDF.GetResource( item.url );
		
		ds.Assert(
					resource, 
					RDF.GetResource("http://"+ item.bucket.name +".s3.amazonaws.com/#name"), 
					RDF.GetLiteral(item.name), 
					true);
	
		ds.Assert(
					resource,
					RDF.GetResource("http://"+ item.bucket.name +".s3.amazonaws.com/#size"),
					RDF.GetLiteral(item.fileSize()),
					true);
		
		ds.Assert(
					resource,
					RDF.GetResource("http://"+ item.bucket.name +".s3.amazonaws.com/#lastModified"),
					RDF.GetLiteral(FB.OBJ.isoDateTime()),
					true);
	
		ds.Assert(
					resource,
					RDF.GetResource("http://"+ item.bucket.name +".s3.amazonaws.com/#etag"),
					RDF.GetLiteral(item.etag),
					true);
	
		ds.Assert(
					resource,
					RDF.GetResource("http://"+ item.bucket.name +".s3.amazonaws.com/#storageClass"),
					RDF.GetLiteral("STANDARD"),	// At the moment there is only STANDARD
					true);
	
		// We should know the ownerID and ownerName, because we probably just uploaded this item
		ds.Assert(
					resource,
					RDF.GetResource("http://"+ item.bucket.name +".s3.amazonaws.com/#ownerID"),
					RDF.GetLiteral("Unknown"),
					true);
					
		ds.Assert(
					resource,
					RDF.GetResource("http://"+ item.bucket.name +".s3.amazonaws.com/#ownerName"),
					RDF.GetLiteral("Unknown"),
					true);					
	
		// Put the new item in a container
		
		var seqResource = RDF.GetResource(item.parentURL());
		var containerUtils = Components.classes["@mozilla.org/rdf/container-utils;1"].
                          		getService(Components.interfaces.nsIRDFContainerUtils);
		var container = containerUtils.MakeSeq(ds, seqResource);
		container.AppendElement(resource);
		
		// Save
	
		var rds = ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
		rds.Flush();
	};

FB.OBJ.RDF.deleteNode =
	function(item) {
	
		var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"]
										.getService(Components.interfaces.nsIRDFService);
		var ds = RDF.GetDataSource("file://"+ item.bucket.storeURL());	
	
		var seqResource = RDF.GetResource(item.parentURL());
		var containerUtils = Components.classes["@mozilla.org/rdf/container-utils;1"].
                          		getService(Components.interfaces.nsIRDFContainerUtils)
		var container = containerUtils.MakeSeq(ds, seqResource);
		
		var resource = RDF.GetResource( item.url );		
		container.RemoveElement(resource, true);
		
		var rds = ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
		rds.Flush();
	}

/**
 * Strings used to create XML requests
 */
FB.OBJ.XML = {};
FB.OBJ.XML.LOGGING_ON = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
FB.OBJ.XML.LOGGING_ON += "<BucketLoggingStatus xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">\n";
FB.OBJ.XML.LOGGING_ON += "<LoggingEnabled>\n";
FB.OBJ.XML.LOGGING_ON += "<TargetBucket>##targetBucket##</TargetBucket>\n";
FB.OBJ.XML.LOGGING_ON += "<TargetPrefix>##targetPrefix##</TargetPrefix>\n";
FB.OBJ.XML.LOGGING_ON += "</LoggingEnabled>\n";
FB.OBJ.XML.LOGGING_ON += "</BucketLoggingStatus>"

FB.OBJ.XML.LOGGING_OFF = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
FB.OBJ.XML.LOGGING_OFF += "<BucketLoggingStatus xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">";
FB.OBJ.XML.LOGGING_OFF += "</BucketLoggingStatus>"
