/*
    Copyright 2009 Graham King <graham@gkgk.org>
    
    This file is part of FireBucket.

    FireBucket is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FireBucket is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FireBucket.  If not, see <http://www.gnu.org/licenses/>.
*/
FB.top = null;	// onSave needs to access this
FB.initBucketLoggingWindow = function() {
	FB.top = new FB.BucketLoggingWindow(FB.account, FB.bucket, FB.callback);
	FB.top.run();
}

FB.onSave = function() {
	FB.top.save();
}

FB.onCancel = function() {
	return true;
}

FB.BucketLoggingWindow = function(account, bucket, callback) {
	this.account = account;
	this.bucket = bucket;
	this.callback = callback;
}

FB.BucketLoggingWindow.prototype = {

	run:
		function() {
		
			function callback(logging) {
			
				if (logging.BucketLoggingStatus.LoggingEnabled !== undefined) {
					isLog.setAttribute('checked', 'true');
					
					var targetBucket = logging.BucketLoggingStatus.LoggingEnabled.TargetBucket;
					var bucketSelector = document.getElementById('fb_log_bucket');
					var allBucketNodes = document.getElementById('fb_bucketMenu').childNodes;
					for (var i=0; i<allBucketNodes.length; i++) {
						var candidate = allBucketNodes[i];
						if ( candidate.getAttribute('label') == targetBucket ) {
							bucketSelector.selectedItem = candidate;
							break;
						}
					}
					
					var targetPrefix = logging.BucketLoggingStatus.LoggingEnabled.TargetPrefix;
					document.getElementById('fb_log_prefix').setAttribute('value', targetPrefix);
				}
				
				_this.onLoggingToggle();
			}
		
			var _this = this;
			
			var isLog = document.getElementById('fb_logging_on');
			isLog.addEventListener('click', function() {_this.onLoggingToggle()}, false);
			
			this.displayBuckets();
			this.bucket.getLogging(callback);
		},
		
	isLoggingOn:
		function() {
			var isLog = document.getElementById('fb_logging_on');
			return isLog.hasAttribute('checked');
		},		
		
	onLoggingToggle:
		function() {
		
			var container = document.getElementById('fb_log_container');
			if (this.isLoggingOn()) {
				container.setAttribute('class', '');
				window.sizeToContent();
			}
			else {
				container.setAttribute('class', 'hidden');
			}
		},
		
	displayBuckets: 
		function() {

			function bucketMenuItem(bucket) {
				var newitem = document.createElement('menuitem');
				newitem.setAttribute('id', bucket.name);
		        newitem.setAttribute('label', bucket.name);
				return newitem;
			}
			
			var _this = this;
			var bucketSelect = document.getElementById('fb_bucketMenu');
			while (bucketSelect.firstChild) {
				bucketSelect.removeChild(bucketSelect.firstChild);
			}

		    for (var i=0; i < this.account.buckets.length; i++) {
		    	var buck = this.account.buckets[i];
				bucketSelect.appendChild( bucketMenuItem(buck) );
		    }

		},
		
	save:
		function() {

			if (this.isLoggingOn()) {
							
				var targetBucket = document.getElementById('fb_log_bucket').selectedItem.label;
				var targetPrefix = document.getElementById('fb_log_prefix').value;
				
				this.bucket.setLoggingOn(targetBucket, targetPrefix);
			}
			else {
				this.bucket.setLoggingOff();
			}

		}
		
}
