/*
    Copyright 2009 Graham King <graham@gkgk.org>
    
    This file is part of FireBucket.

    FireBucket is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FireBucket is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FireBucket.  If not, see <http://www.gnu.org/licenses/>.
*/
FB.initCreateBucketWindow = function() {
	var b = new FB.CreateBucketWindow(FB.account, FB.callback);
	b.run();
}

FB.CreateBucketWindow = function(account, callback) {
	this.account = account;
	this.callback = callback;
}

FB.CreateBucketWindow.prototype = {

	run:
		function() {
			var _this = this;

			var checkButton = document.getElementById('fb_check_bucket_name');
			checkButton.addEventListener('command', function() {_this.checkName()}, false);
			
			var createButton = document.getElementById('fb_create_bucket');
			createButton.addEventListener('command', function() {_this.createBucket()}, false);
			
			document.getElementById('fb_bucket_name').focus();
		},
		
	checkName:
		function() {
		
			function onAvailabilityCheck(isAvailable) {

				isAvailableDisplay.setAttribute('value', (isAvailable ? 'Yes':'No'));
				isAvailableDisplay.setAttribute('class', (isAvailable? 'good':'bad'));
				
				if ( isAvailable ) {
					var createButton = document.getElementById('fb_create_bucket');
					createButton.setAttribute('disabled', false);
				}
			}
		
			var name = document.getElementById('fb_bucket_name').value;

			// Is it valid?
			var isValidDisplay = document.getElementById('fb_valid_bucket_name');
			if ( this.isValidBucketName(name) ) {
				isValidDisplay.setAttribute('value', 'Yes');
				isValidDisplay.setAttribute('class', 'good');
			}
			else {
				isValidDisplay.setAttribute('value', 'No');
				isValidDisplay.setAttribute('class', 'bad');				
				return;
			}

			// Does it already exist?
			var isAvailableDisplay = document.getElementById('fb_bucket_name_available');
			isAvailableDisplay.setAttribute('value', 'Checking...');
			
			this.account.isBucketNameAvailable(name, onAvailabilityCheck);

		},

	isValidBucketName:
		function(candidate) {
			// This is actually a little stricter than 'valid' - it enforces the DNS
			//  recommendations from here:
			// http://docs.amazonwebservices.com/AmazonS3/2006-03-01/index.html?UsingBucket.html

			// Use 3 to 63 characters
			// Use only lower case letters, numbers, '.', and '-'.
			var BUCKET_NAME_RE = new RegExp("^[a-z0-9\.\-]{3,63}$");
			if ( ! BUCKET_NAME_RE.test(candidate) ) {
				return false;
			}
			
			//Don't start or end the bucket name with '.' 
			if ( candidate[0] == '.' || candidate[candidate.length-1] == '.' ) {
				return false;
			}
			
			// Don't follow or precede a '.' With a '-'.
			if ( candidate.indexOf('.-') != -1 || candidate.indexOf('-.') != -1 ) {
				return false;
			}

			return true;
		},
		
	createBucket:
		function() {
		
			function onComplete() {
				FB.OBJ.status('New bucket '+ name +' created');
				if (location == 'EU') {
					alert('EU buckets take about 5 minutes to propagate. '+
							'During that time all requests on the bucket will fail with an Error 403.');
				}
				_this.callback();
				window.close();
			}

			var _this = this;
			
			var name = document.getElementById('fb_bucket_name').value;
			var location = document.getElementById('fb_location').selectedItem.label;
			var acl = document.getElementById('fb_acl').selectedItem.label;
				
			var locationConstraint = '';		
			if (location == 'EU') {
				locationConstraint = "<CreateBucketConfiguration><LocationConstraint>EU</LocationConstraint></CreateBucketConfiguration>";
			}
						
			this.account.createBucket(name, locationConstraint, acl, onComplete);
		}
	
		
}
