/*
    Copyright 2009 Graham King <graham@gkgk.org>
    
    This file is part of FireBucket.

    FireBucket is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FireBucket is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FireBucket.  If not, see <http://www.gnu.org/licenses/>.
*/
if (typeof FB == "undefined" || !FB) {
	var FB = {};
}

FB.open = function() {
    gBrowser.selectedTab = gBrowser.addTab('chrome://firebucket/content/firebucket.xul');
}

// This is only a global object so we can play with it in the Firebug console, no other need
FB.top = null;
FB.main = function() {

	FB.top = new FB.DisplayManager();
	FB.top.run();
}

FB.DisplayManager = function() {

	this.account = 'none yet';
	this.activeBucket = null;
	this.activeItem = null;
	this.dragDropObserver = null;
	
    this.isHandlersSet = false;
    
	FB.OBJ.setup();
}

FB.DisplayManager.prototype = {

	run:
		function() {
            
			FB.OBJ.status('Starting...');
			var _this = this;

			var key = FB.OBJ.getCharPref('key');
			var secret = FB.OBJ.getCharPref('secret');
		
			var hasKey = (key != null) && (key != '');
			var hasSecret = (secret != null) && (secret != '');
		
			if (hasKey && hasSecret) {

				this.attachEventHandlers();
				
				document.getElementById('fb_loadbuckets').setAttribute('disabled', false);

				this.account = new FB.OBJ.Account(key, secret);
			    this.account.loadBuckets(function() {_this.displayBuckets()}, false);
			    
			    FB.OBJ.account = this.account;
			}
			else {
				this.onSettings();
			}		
		},

	attachEventHandlers:
		function() {
            
            if (this.isHandlersSet) {
                return;
            }
                        
			var _this = this;
		
			var settingsButton = document.getElementById('fb_settings');
			settingsButton.addEventListener('command', function() {_this.onSettings()}, false);
			
			var createBucketButton = document.getElementById('fb_createbucket');
			createBucketButton.addEventListener('command', function(){_this.showCreateBucketDialog()}, false);
			
			var clearCacheButton = document.getElementById('fb_clearcache');
			clearCacheButton.addEventListener('command', function() {_this.clearCache()}, false);
			
			var refreshButton = document.getElementById('fb_loadbuckets');
			refreshButton.addEventListener('command', function() {_this.refreshBuckets()}, false);
			
			var refreshItemsButton = document.getElementById('fb_refreshitems');
			refreshItemsButton.addEventListener('command', function() {_this.refreshItems()}, false);
			
			var loggingButton = document.getElementById('fb_bucketlogging');
			loggingButton.addEventListener('command', function() {_this.onLogging()}, false);
			
			var aclButton = document.getElementById('fb_bucketacl');
			aclButton.addEventListener('command', function() {_this.onACL()}, false);
			
			var deleteBucketButton = document.getElementById('fb_deletebucket');
			deleteBucketButton.addEventListener('command', function() {_this.onDeleteBucket()}, false);
			
			var deleteItemButton = document.getElementById('fb_delete_item');
			deleteItemButton.addEventListener('command', function() {_this.deleteItem()}, false);
			
			var openItemButton = document.getElementById('fb_open_item');
			openItemButton.addEventListener('command', function() {_this.openItemInTab()}, false);
			
			var updateHeadersButton = document.getElementById('fb_update_headers');
			updateHeadersButton.addEventListener('command', function() {_this.onUpdateItemHeaders()}, false);
			
			var addHeaderButton = document.getElementById('fb_add_header');
			addHeaderButton.addEventListener('command', function() {_this.onAddHeader()}, false);
            
            this.isHandlersSet = true;
		},

	activateBucket:
		function(bucket) {
			this.activeBucket = bucket;
			this.activeItem = null;
			
			document.getElementById('fb_delete_item').setAttribute('disabled', true);
			document.getElementById('fb_open_item').setAttribute('disabled', true);
			
			var refreshItemsButton = document.getElementById('fb_refreshitems');
			refreshItemsButton.setAttribute('disabled', false);
		
			var loggingButton = document.getElementById('fb_bucketlogging');
			loggingButton.setAttribute('disabled', false);
			
			var aclButton = document.getElementById('fb_bucketacl');
			aclButton.setAttribute('disabled', false);
			
			var locationLabel = document.getElementById('fb_bucketlocation');
			locationLabel.setAttribute('value', 'Location: '+ bucket.location);
		},

	onLogging:
		function() {
			openDialog('chrome://firebucket/content/bucket_logging.xul', 
						'Bucket Logging', 
						"", 
						FB.OBJ, 
						this.account,
						this.activeBucket,
						function() {FB.OBJ.log('Dialog complete')}
						);
		},

	onACL:
		function() {
			openDialog('chrome://firebucket/content/bucket_acl.xul', 
						'Access Control List', 
						"", 
						FB.OBJ, 
						this.account,
						this.activeBucket,
						function() {FB.OBJ.log('Dialog complete')}
						);
		},

	onSettings: 
		function() {
                        
			var _this = this;
			openDialog(
						'chrome://firebucket/content/preferences.xul', 
						'FireBucket Settings',
						"",
						function() {_this.run()}
						);
		},

	onDeleteBucket:
		function() {

			var _this = this;
			if ( confirm('Delete the '+ this.activeBucket.name +' bucket?') ) {
				this.activeBucket.delete(function() {_this.refreshBuckets()});
			}
		},

	refreshBuckets:
		function() {
			var _this = this;
		    this.account.loadBuckets(
		    	function() {_this.displayBuckets(); FB.OBJ.status('Bucket list reloaded')}, 
		    	true);
		},

	showCreateBucketDialog:
		function() {
			var _this = this;
			openDialog('chrome://firebucket/content/create_bucket.xul', 
						'Create bucket', 
						"", 
						FB.OBJ, 
						this.account,
						function() {_this.refreshBuckets()}
						);
		},

	startDragDropObserver:
		function() {
		
			if ( this.dragDropObserver != null ) {
				return;
			}

			var _this = this;
			this.dragDropObserver = new FB.DragDropObserver(this);
			
			var dropTarget = document.getElementById('fb_droptarget');
			dropTarget.addEventListener('dragdrop', 
										function(event) {
											nsDragAndDrop.drop(event,_this.dragDropObserver)
										}, 
										false);
			dropTarget.addEventListener('draggesture', 
										function(event) {
											nsDragAndDrop.startDrag(event,_this.dragDropObserver)
										}, 
										false);
			dropTarget.addEventListener('dragover', 
										function(event) {
											nsDragAndDrop.dragOver(event,_this.dragDropObserver)
										}, 
										false);
			dropTarget.addEventListener('dragexit', 
										function(event) {
											nsDragAndDrop.dragExit(event,_this.dragDropObserver)
										}, 
										false);

			document.getElementById('fb_droptarget_desc').
						setAttribute('value', 'Drop files here to upload');
		},
		
	onTreeSelect:
		function(event) {

			var tree = document.getElementById(event.target.id);
			if (tree.currentIndex < 0) {
				return;
			}
			var url = tree.view.getResourceAtIndex(tree.currentIndex).Value;
			//url = escape(url).replace(/%3A/, ':');
			
			document.getElementById('fb_delete_item').setAttribute('disabled', false);
			document.getElementById('fb_open_item').setAttribute('disabled', false);
			
			// reset the download progress meter and show it
			document.getElementById("fb_dl_progress").setAttribute('value', 0);	
			document.getElementById('fb_dl_label').setAttribute('value', 'Downloading...');
			document.getElementById('fb_dl').setAttribute('class', '');
			document.getElementById('fb_display_panel').setAttribute('src', 'about:blank');			
			
			var _this = this;
			this.activeBucket.loadItem(url, function(item) {
												_this.activeItem = item;
												_this.displayItem(item)
											});
		},

	displayItem:
		function(item) {

			function row(key, value) {

				var keyNode = document.createElement('label');
				keyNode.setAttribute('value', key);
				
				if (FB.OBJ.isAllowedHeader(key)) {
					var valueNode = document.createElement('textbox');
					valueNode.setAttribute('flex', 1);
				}
				else {
					var valueNode = document.createElement('label');
					valueNode.setAttribute('crop', 'end');
				}
				valueNode.setAttribute('value', FB.OBJ.trim(value));
				
				var row = document.createElement('row');
				row.appendChild(keyNode);
				row.appendChild(valueNode);			
				return row;
			}

			var _this = this;
			
			document.getElementById('fb_dl').setAttribute('class', 'hidden');
			
			var fullPath = item.cacheFileURL();
			document.getElementById('fb_display_panel').setAttribute('src', fullPath);
			
			var headersContainer = document.getElementById('fb_headers_rows');
			while (headersContainer.firstChild) {
				headersContainer.removeChild(headersContainer.firstChild);
			}
			
			var headers = item.headersMap();
			var editableHeaders = [];
			for (var key in headers) {
				var rowNode = row(key, headers[key]);
				
				if (FB.OBJ.isAllowedHeader(key)) {
					editableHeaders.push(rowNode);
				}
				else {
					headersContainer.appendChild(rowNode);
				}
			}
			
			// Editable headers display after all the read only headers
			for (var i=0; i<editableHeaders.length; i++) {
				headersContainer.appendChild(editableHeaders[i]);
			}
		},

	onAddHeader:	// Add a header to the current active item
		function() {
			var keyNode = document.createElement('textbox');
			keyNode.setAttribute('flex', 1);
			
			var valueNode = document.createElement('textbox');
			valueNode.setAttribute('flex', 1);
			
			var row = document.createElement('row');
			row.appendChild(keyNode);
			row.appendChild(valueNode);
		
			var headersContainer = document.getElementById('fb_headers_rows');
			headersContainer.appendChild(row);
		},

	onUpdateItemHeaders:
		function() {
		
			function callback() {
				FB.OBJ.status('Header update complete');
			}

			var headersContainer = document.getElementById('fb_headers_rows');
			var headers = {}
			for (var i=0; i < headersContainer.childNodes.length; i++) {
				var row = headersContainer.childNodes[i];
				var name = FB.OBJ.trim(row.childNodes[0].value);
				if (name == '') {
					continue;
				}
				var value = FB.OBJ.trim(row.childNodes[1].value);
				
				if (FB.OBJ.isAllowedHeader(name)) {
					headers[name] = value;
				}
			}
			
			FB.OBJ.status('Updating headers...');
			this.activeItem.setHeaders(headers, callback);
		},

	displayBuckets: 
		function() {

			function bucketMenuItem(bucket) {
				var newitem = document.createElement('menuitem');
				newitem.setAttribute('id', bucket.name);
		        newitem.setAttribute('label', bucket.name);
		        newitem.addEventListener('command', function(event) {_this.onBucketSelect(event)}, false);
				return newitem;
			}
			
			var _this = this;
			var bucketSelect = document.getElementById('fb_bucketMenu');
			while (bucketSelect.firstChild) {
				bucketSelect.removeChild(bucketSelect.firstChild);
			}

		    for (var i=0; i < this.account.buckets.length; i++) {
		    	var buck = this.account.buckets[i];
				bucketSelect.appendChild( bucketMenuItem(buck) );
		    }

			FB.OBJ.status('');
		},

	onBucketSelect: 
		function (event) {
		
		    var bucketName = event.target.id;
			var bucket = this.getBucketByName(bucketName);
			
			this.activateBucket(bucket);			
			this.startDragDropObserver();
			
			var _this = this;
			bucket.listItems( function() {_this.displayItems()}, (forceReload=false) );
		},

	refreshItems:
		function() {
		
			this.activeItem = null;
			document.getElementById('fb_delete_item').setAttribute('disabled', true);
			document.getElementById('fb_open_item').setAttribute('disabled', true);
			document.getElementById('fb_items').view.selection.clearSelection();

			var _this = this;
			this.activeBucket.listItems( function() {_this.displayItems()}, (forceReload=true) );
		},	
	
	getBucketByName:
		function(bucketName) {
			for (var i=0; i<this.account.buckets.length; i++) {
				var buck = this.account.buckets[i];
				if (buck.name == bucketName) {
					return buck;
				}
			}
			return null;
		},
	
	displayItems:
		function() {

			function setCell(key) {
				var cell = document.getElementById('fb_'+ key +'_cell');
				cell.setAttribute('label', 'rdf:http://'+ bucket.name +'.s3.amazonaws.com/#'+ key);
			}

			var bucket = this.activeBucket;
	
			setCell('name');
			setCell('size');
			setCell('lastModified');
			setCell('etag');
			setCell('storageClass');
			setCell('ownerID');
			setCell('ownerName');

			var tree = document.getElementById('fb_items');
			tree.setAttribute('datasources', FileIO.path(FB.OBJ.storeDir()) +'/'+ bucket.name +'.rdf');
			tree.setAttribute('ref', 'http://'+ bucket.name +'.s3.amazonaws.com/all');
	
			tree.setAttribute('disabled', false);
			var _this = this;
			tree.addEventListener('select', function(event) {_this.onTreeSelect(event)}, false);

			FB.OBJ.status('Reloading data...');
			
			setTimeout( function() {tree.builder.refresh(); FB.OBJ.status('Displaying...');}, 1);
			
			setTimeout(
				function(){ 
				
					tree.builder.rebuild();
					
					if (tree.view.rowCount == 0) {
						var deleteBucketButton = document.getElementById('fb_deletebucket');
						deleteBucketButton.setAttribute('disabled', false);
					}
					
					FB.OBJ.status(''); 
				}, 
			200);
			
		},

	redisplay:
		function() {
			var tree = document.getElementById('fb_items');
			tree.builder.refresh(); 
		},

	clearCache:	
		function() {
			DirIO.unlink(FB.OBJ.cacheDir(), true);
			FB.OBJ.status('Cache cleared');
		},
		
	openItemInTab:
		function() {
			FB.OBJ.open(this.activeItem.url);
		},
		
	deleteItem:
		function() {
		
			function deleteDone() {
				_this.activeItem = null;
				document.getElementById('fb_delete_item').setAttribute('disabled', true);
				document.getElementById('fb_open_item').setAttribute('disabled', true);
				//_this.refreshItems();
			}

			var _this = this;	
			if ( confirm('Delete '+ this.activeItem +'?') ) {
				this.activeItem.delete(deleteDone);
			}
		
		}

};

FB.DragDropObserver = function(displayManager) {
	var _this = this;
	
	var uploadButton = document.getElementById('fb_upload_button');
	uploadButton.addEventListener('command', function() {_this.upload()}, false);
	
	var cancelButton = document.getElementById('fb_cancelupload_button');
	cancelButton.addEventListener('command', function() {_this.cancel()}, false);
	
	this.displayManager = displayManager;
	this.file = null;
	
	this.extraHeaderCount = 0;
}

FB.DragDropObserver.prototype = {

	cancel:
		function() {
		
			document.getElementById('fb_upload_info').setAttribute('class', 'hidden');
				
			var topContainer = document.getElementById('fb_upload_list');
			while (topContainer.firstChild) {
				topContainer.removeChild(topContainer.firstChild);
			}
			
			var uploadCaption = document.getElementById('fb_upload_caption');
			uploadCaption.setAttribute('label', '');
			
			document.getElementById('fb_droptarget').setAttribute('class', '');
			
			this.file= null;
			this.extraHeaderCount = 0;
		},

	upload:
		function() {
		
			function onUploadComplete() {
				_this.cancel();
				_this.displayManager.redisplay();
			}

			var _this = this;

			var key = document.getElementById('fb_upfile_key').value;
			var mime = document.getElementById('fb_upfile_mime').value;
			
			var extraHeaders = {};
			for (var i=0; i<this.extraHeaderCount; i++) {
				var check = document.getElementById('fb_check_extra'+ i);
				
				if (check.hasAttribute('checked')) {
					var header = document.getElementById('fb_label_extra'+ i).value;
					var value = document.getElementById('fb_upfile_extra'+ i).value;
					extraHeaders[header] = value;
				}
			}
			
			FB.OBJ.status('Uploading...');
			this.displayManager.activeBucket.upload(key, mime, extraHeaders, this.file, onUploadComplete);
		},

	onDragOver:
		function(event, flavour, session) {
		},
		
	onDrop:
		function(event, dropData, session)  {

			function labelValueNode(label, value, id, optional) {
					
					var container = document.createElement('hbox');

					if (optional) {
						var checkNode = document.createElement('checkbox');
						checkNode.setAttribute('id', 'fb_check_'+ id);
						checkNode.setAttribute('checked', 'true');
						checkNode.addEventListener('click', function(event) {_this.onToggleHeader(event.target.id)}, false);
						
						container.appendChild(checkNode);
					}

					var labelNode = document.createElement('label');
					labelNode.setAttribute('value', label)
					container.appendChild(labelNode);
					
					var textbox = document.createElement('textbox');
					textbox.setAttribute('id', 'fb_upfile_'+ id );
					textbox.setAttribute('flex', 1);
					textbox.setAttribute('value', value);
					container.appendChild(textbox);
		
					return container;			
			}

			var _this = this;
			
			this.file = dropData.data;

			var uploadCaption = document.getElementById('fb_upload_caption');
			uploadCaption.setAttribute('label', this.file.leafName);

			var topContainer = document.getElementById('fb_upload_list');
			
			var initialKey = null;
			if (this.file.path[0] == '/') {	// *nx OS
				initialKey = this.file.path.slice(1, this.file.path.length);
			}
			else if (this.file.path[1] == ':') {	// Windows OS
				initialKey = this.file.path.slice(3, this.file.path.length);
				initialKey = initialKey.replace(/\\/g, '/');
			}
			else {	// Not sure
				initialKey = this.file.path;
			}
			
			var keyNode = labelValueNode(
										'Key:', 
										initialKey, 
										'key',
										false);
			topContainer.appendChild(keyNode);
			
			var mimeNode = labelValueNode(
										'Content-Type:', 
										FB.OBJ.mimeType(this.file.leafName), 
										'mime',
										false);
			topContainer.appendChild(mimeNode);

			var addButton = document.createElement('button');
			addButton.setAttribute('label', 'Add header or meta-data');
			addButton.addEventListener('command', function() {_this.addHeader('', '')}, false);
			topContainer.appendChild(addButton);
			
			this.addHeader('Cache-Control',	'max-age=315360000');
			this.addHeader('Expires', 'Mon, 27 Aug 2018');
			
			document.getElementById('fb_droptarget').setAttribute('class', 'hidden');
			document.getElementById('fb_upload_info').setAttribute('class', '');
		},
		
	addHeader:
		function(labelText, valueText) {
			var _this = this;
			
			var container = document.createElement('hbox');
	
			var checkNode = document.createElement('checkbox');
			checkNode.setAttribute('id', 'fb_check_extra'+ this.extraHeaderCount);
			checkNode.setAttribute('checked', 'true');
			checkNode.addEventListener('click', function(event) {_this.onToggleHeader(event.target.id)}, false);
			container.appendChild(checkNode);

			var labelNode = document.createElement('textbox');
			labelNode.setAttribute('id', 'fb_label_extra'+ this.extraHeaderCount);
			labelNode.setAttribute('value', labelText)
			container.appendChild(labelNode);
			
			var textbox = document.createElement('textbox');
			textbox.setAttribute('id', 'fb_upfile_extra'+ this.extraHeaderCount);
			textbox.setAttribute('flex', 1);
			textbox.setAttribute('value', valueText);
			container.appendChild(textbox);

			var topContainer = document.getElementById('fb_upload_list');
			topContainer.insertBefore(container, topContainer.lastChild);
			
			this.extraHeaderCount++;
		},
		
	getSupportedFlavours:
		function()  {
			var flavours = new FlavourSet();
	        flavours.appendFlavour("application/x-moz-file", "nsIFile");
	        flavours.appendFlavour("text/unicode");
	        flavours.appendFlavour("text/plain");
	        flavours.appendFlavour("text/x-moz-url");        
	        flavours.appendFlavour("FileContents");
    	    return flavours;
		},
		
	onToggleHeader:
		function(checkId) {
			var id = checkId.split('_')[2];
			var labelbox = document.getElementById('fb_label_'+ id);
			var textbox = document.getElementById('fb_upfile_'+ id);
			
			if (textbox.hasAttribute('disabled')) {
				labelbox.removeAttribute('disabled');
				textbox.removeAttribute('disabled');
			}
			else {
				labelbox.setAttribute('disabled', true);
				textbox.setAttribute('disabled', true);
			}
		}
};
